{
Copyright (c) Microsoft Corporation. All rights reserved.
Module Name:
    dbghelp.h
Abstract:
    This module defines the prototypes and constants required for the image
    help routines.
    Contains debugging support routines that are redistributable.
}

//Parts of dbghelp.h ported to Object Pascal by Alex Shovkoplyas, VE3NEA

unit DbgHelp;

interface

uses
  Windows;

{$MINENUMSIZE 4}


const
  DbgHelpDll = 'DbgHelp';

  SYMOPT_CASE_INSENSITIVE           = $00000001;
  SYMOPT_UNDNAME                    = $00000002;
  SYMOPT_DEFERRED_LOADS             = $00000004;
  SYMOPT_NO_CPP                     = $00000008;
  SYMOPT_LOAD_LINES                 = $00000010;
  SYMOPT_OMAP_FIND_NEAREST          = $00000020;
  SYMOPT_LOAD_ANYTHING              = $00000040;
  SYMOPT_IGNORE_CVREC               = $00000080;
  SYMOPT_NO_UNQUALIFIED_LOADS       = $00000100;
  SYMOPT_FAIL_CRITICAL_ERRORS       = $00000200;
  SYMOPT_EXACT_SYMBOLS              = $00000400;
  SYMOPT_ALLOW_ABSOLUTE_SYMBOLS     = $00000800;
  SYMOPT_IGNORE_NT_SYMPATH          = $00001000;
  SYMOPT_INCLUDE_32BIT_MODULES      = $00002000;
  SYMOPT_PUBLICS_ONLY               = $00004000;
  SYMOPT_NO_PUBLICS                 = $00008000;
  SYMOPT_AUTO_PUBLICS               = $00010000;
  SYMOPT_NO_IMAGE_SEARCH            = $00020000;
  SYMOPT_SECURE                     = $00040000;
  SYMOPT_NO_PROMPTS                 = $00080000;
  SYMOPT_OVERWRITE                  = $00100000;
  SYMOPT_IGNORE_IMAGEDIR            = $00200000;
  SYMOPT_FLAT_DIRECTORY             = $00400000;
  SYMOPT_FAVOR_COMPRESSED           = $00800000;
  SYMOPT_ALLOW_ZERO_ADDRESS         = $01000000;
  SYMOPT_DISABLE_SYMSRV_AUTODETECT  = $02000000;
  SYMOPT_DEBUG                      = $80000000;

  MAX_SYM_NAME = 2000;


type   
  TSymType = (SymNone, SymCoff, SymCv, SymPdb, SymExport, SymDeferred, SymSym,
    SymDia, SymVirtual, NumSymTypes);


  PImagehlpModule64 = ^TImagehlpModule64;
  TImagehlpModule64 = record
    SizeOfStruct:           DWORD;              // set to sizeof(IMAGEHLP_MODULE64)
    BaseOfImage:            Int64;              // base load address of module
    ImageSize:              DWORD;              // virtual size of the loaded module
    TimeDateStamp:          DWORD;              // date/time stamp from pe header
    CheckSum:               DWORD;              // checksum from the pe header
    NumSyms:                DWORD;              // number of symbols in the symbol table
    SymType:                TSymType;           // type of symbols loaded
    ModuleName: array[0..31] of Char;           // module name
    ImageName: array[0..255] of Char;           // image name
    LoadedImageName: array[0..255] of Char;     // symbol file name
    LoadedPdbName: array[0..255] of Char;       // pdb file name
    CVSig:                  DWORD;              // Signature of the CV record in the debug directories
    CVData: array[0..(MAX_PATH * 3)-1] of Char; // Contents of the CV record
    PdbSig:                 DWORD;              // Signature of PDB
    PdbSig70:               TGUID;              // Signature of PDB (VC 7 and up)
    PdbAge:                 DWORD;              // DBI age of pdb
    PdbUnmatched:           BOOL;               // loaded an unmatched pdb
    DbgUnmatched:           BOOL;               // loaded an unmatched dbg
    LineNumbers:            BOOL;               // we have line number information
    GlobalSymbols:          BOOL;               // we have internal symbol information
    TypeInfo:               BOOL;               // we have type information
    SourceIndexed:          BOOL;               // pdb supports source server
    Publics:                BOOL;               // contains public symbols
    end;


  PImagehlpModule = ^TImagehlpModule;
  TImagehlpModule = record
    SizeOfStruct:           DWORD;              // set to sizeof(IMAGEHLP_MODULE64)
    BaseOfImage:            DWord;              // base load address of module
    ImageSize:              DWORD;              // virtual size of the loaded module
    TimeDateStamp:          DWORD;              // date/time stamp from pe header
    CheckSum:               DWORD;              // checksum from the pe header
    NumSyms:                DWORD;              // number of symbols in the symbol table
    SymType:                TSymType;           // type of symbols loaded
    ModuleName: array[0..31] of Char;           // module name
    ImageName: array[0..255] of Char;           // image name
    LoadedImageName: array[0..255] of Char;     // symbol file name
    end;


  PImageHlpSymbol = ^TImageHlpSymbol;
  TImageHlpSymbol = record
    SizeOfStruct:  DWORD;      // set to sizeof(IMAGEHLP_SYMBOL)
    Address:       DWORD;      // virtual address including dll base address
    Size:          DWORD;      // estimated size of symbol, can be zero
    Flags:         DWORD;      // info about the symbols, see the SYMF defines
    MaxNameLength: DWORD;      // maximum size of symbol name in 'Name'
    Name: array[0..0] of CHAR; // symbol name (null terminated string)
    end;

    
  TImageHlpSymbolPackage = record
    sym: TImageHlpSymbol;
    name: array[0..MAX_SYM_NAME] of Char;
    end;


  PImageHelpLine = ^TImageHelpLine;
  TImageHelpLine = record
    SizeOfStruct: DWORD;   // set to sizeof(IMAGEHLP_LINE)
    Key:          Pointer; // internal
    LineNumber:   DWORD;   // line number in file
    FileName:     PChar;   // full filename
    Address:      DWORD;   // first instruction of line
    end;


function SymInitialize(hProcess: THandle; UserSearchPath: PChar;
  fInvadeProcess: LongBool): LongBool; stdcall; external DbgHelpDll;

function SymCleanup(hProcess: THandle): LongBool; stdcall; external DbgHelpDll;

function SymGetOptions:DWord; stdcall; external DbgHelpDll;

procedure SymSetOptions(SymOptions: DWord); stdcall; external DbgHelpDll;

function SymSetSearchPath(hProcess: THandle; SearchPath: PChar): LongBool;
  stdcall; external DbgHelpDll;

function SymLoadModule(hProcess: THandle; hFile: THandle; ImageName: PChar;
  ModuleName: PChar; BaseOfDll: DWord; SizeOfDll: DWord): DWord;
  stdcall; external DbgHelpDll;

function SymUnloadModule(hProcess: THandle; BaseOfDll: DWord): LongBool;
  stdcall; external DbgHelpDll;

function SymGetModuleInfo(hProcess: THandle; dwAddr: DWord;
  out ModuleInfo: TImagehlpModule): LongBool; stdcall; external DbgHelpDll;


function SymGetSymFromAddr(hProcess: THandle; dwAddr: DWord;
  pdwDisplacement: PDword; var Symbol: TImageHlpSymbol): LongBool;
  stdcall; external DbgHelpDll;

function SymGetLineFromAddr(hProcess: THandle; dwAddr: DWORD;
  pdwDisplacement: PDWORD; var Line: TImageHelpLine): LongBool;
  stdcall; external DbgHelpDll;

function SymGetSymFromName(hProcess: THandle; Name: PChar;
  var Symbol: TImageHlpSymbol): LongBool; stdcall; external DbgHelpDll;

function SymGetSymNext(hProcess: THandle; var Symbol: TImageHlpSymbol): LongBool;
  stdcall; external DbgHelpDll;



implementation

end.

