//------------------------------------------------------------------------------
//The contents of this file are subject to the Mozilla Public License
//Version 1.1 (the "License"); you may not use this file except in compliance
//with the License. You may obtain a copy of the License at
//http://www.mozilla.org/MPL/ Software distributed under the License is
//distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
//or implied. See the License for the specific language governing rights and
//limitations under the License.
//
//The Original Code is Main.pas.
//
//The Initial Developer of the Original Code is Alex Shovkoplyas, VE3NEA.
//Portions created by Alex Shovkoplyas are
//Copyright (C) 2008 Alex Shovkoplyas. All Rights Reserved.
//------------------------------------------------------------------------------
unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, ComCtrls, Math, ToolWin, ImgList, StdCtrls, VirtualTrees,
  AppEvnts, RunApp, Samplr, PerfData, TreeFrm, ThrLst, Sett, PathUtil,
  DbgSrc, ViewFrm, SynHighlighterAsm, SynHighlighterPas, ShellApi,
  SynEditHighlighter, SynHighlighterCpp, FileCtrl, IniFiles;

const
  WM_SAMPLING_STOPPED = WM_USER + 15;


type
  TProfilerState = (psIdle, psWaiting, psSampling, psPaused);


  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    ExitMNU: TMenuItem;
    StatusBar1: TStatusBar;
    TabControl1: TTabControl;
    Splitter1: TSplitter;
    Panel1: TPanel;
    TreePanel: TPanel;
    View1: TMenuItem;
    VerticalSplitMNU: TMenuItem;
    HorisontalSplitMNU: TMenuItem;
    N1: TMenuItem;
    SettingsMNU: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    ImageList1: TImageList;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton2: TToolButton;
    MruMenu: TPopupMenu;
    NoRecentFilesMNU: TMenuItem;
    Panel4: TPanel;
    Label1: TLabel;
    ComboBox1: TComboBox;
    ApplicationEvents1: TApplicationEvents;
    Timer1: TTimer;
    Profile1: TMenuItem;
    StartMNU: TMenuItem;
    PauseMNU: TMenuItem;
    StopMNU: TMenuItem;
    N2: TMenuItem;
    ExpandTreeMNU: TMenuItem;
    CollapseTreeMNU: TMenuItem;
    TreeFrame1: TTreeFrame;
    Panel5: TPanel;
    SrcPanel: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    ViewFrame1: TViewFrame;
    Contents1: TMenuItem;
    N3: TMenuItem;
    procedure ExitMNUClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SettingsMNUClick(Sender: TObject);
    procedure VerticalSplitMNUClick(Sender: TObject);
    procedure HorisontalSplitMNUClick(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure StartMNUClick(Sender: TObject);
    procedure PauseMNUClick(Sender: TObject);
    procedure StopMNUClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure TabControl1Change(Sender: TObject);
    procedure TreePanelResize(Sender: TObject);
    procedure ExpandTreeMNUClick(Sender: TObject);
    procedure CollapseTreeMNUClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure NoRecentFilesMNUClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Contents1Click(Sender: TObject);
  private
    FState: TProfilerState;

    function StartApp: boolean;
    procedure StopApp;
    procedure StartSampling;
    procedure PauseSampling;
    procedure ResumeSampling;

    procedure TerminateEvent(Sender: TObject);
    procedure DebugEvent(Sender: TObject);

    procedure SetPanelLayout(AVert: boolean; ASplitPos: integer = 0);
    procedure SetState(const Value: TProfilerState);
    procedure ShowData;
    procedure StopSampling;

    procedure ModuleEvent(Sender: TObject; ModuleName: string; Percent: Single);
    procedure WmSamplingStopped(var Msg: TMessage); message WM_SAMPLING_STOPPED;
    procedure SaveToIni;
    procedure LoadFromIni;
  public
    App: TRunApplication;
    Spl: TSampler;
    Perf: TPerformanceData;
    Settings: TSettings;
    Mru: TMruList;
    StartTime, TimeLeft: TDateTime;

    property State: TProfilerState read FState write SetState;
  end;


var
  MainForm: TMainForm;




implementation

uses SettDlg, ProgrDlg;

{$R *.DFM}

//------------------------------------------------------------------------------
//                               init
//------------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
begin
  App := TRunApplication.Create;
  App.OnTerminate := TerminateEvent;
  App.OnDebugEvent := DebugEvent;

  Perf := TPerformanceData.Create;
  Perf.OnProcessModule := ModuleEvent;

  Spl := TSampler.Create;
  if not Spl.CheckInitialized then begin Application.Terminate; Exit; end;

  State := psIdle;

  Mru := TMruList.Create;
  Mru.Load(ExtractFilePath(ParamStr(0)) + 'MRU.ini');
  Mru.ToMenu(NoRecentFilesMNU);
  Settings := Mru[0];

  ViewFrame1.Finder.BorToolIndex := 0;
  Application.HintHidePause := 10000;
  LoadFromIni;
end;


procedure TMainForm.FormDestroy(Sender: TObject);
begin
  StopApp;

  SaveToIni;
  Mru.Save(ExtractFilePath(ParamStr(0)) + 'MRU.ini');

  App.Free;
  Perf.Free;
  Spl.Free;
  Mru.Free;
end;






//------------------------------------------------------------------------------
//                               menu
//------------------------------------------------------------------------------
procedure TMainForm.ExitMNUClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.SettingsMNUClick(Sender: TObject);
begin
  SettingsDialog.ShowModal;
end;

procedure TMainForm.VerticalSplitMNUClick(Sender: TObject);
begin
  SetPanelLayout(true);
end;

procedure TMainForm.HorisontalSplitMNUClick(Sender: TObject);
begin
  SetPanelLayout(false);
end;

procedure TMainForm.StartMNUClick(Sender: TObject);
begin
  if StartApp then
    begin
    Mru.Add(Settings);
    Mru.ToMenu(NoRecentFilesMNU);
    end;
end;

procedure TMainForm.PauseMNUClick(Sender: TObject);
begin
  if State = psPaused then ResumeSampling else PauseSampling;
end;

procedure TMainForm.StopMNUClick(Sender: TObject);
begin
  StopApp;
end;


procedure TMainForm.ExpandTreeMNUClick(Sender: TObject);
begin
  TreeFrame1.Tv.FullExpand;
end;


procedure TMainForm.CollapseTreeMNUClick(Sender: TObject);
begin
  if TabControl1.TabIndex = 0
    then TreeFrame1.Tv.FullCollapse
    else TreeFrame1.ExpandFirstLevel;
end;


procedure TMainForm.NoRecentFilesMNUClick(Sender: TObject);
begin
  Settings := Mru[(Sender as TMenuItem).Tag];
  //SettingsDialog.ShowModal;
end;


procedure TMainForm.Contents1Click(Sender: TObject);
var
  HelpName: TFileName;
begin
  HelpName := ChangeFileExt(ParamStr(0), '.chm');
  ShellExecute(GetDesktopWindow, 'open', PChar(HelpName), '', '', SW_SHOWNORMAL);
end;


procedure TMainForm.About1Click(Sender: TObject);
begin
 MessageDlg('           QRQ Profiler'#13#13'by Alex Shovkoplyas, VE3NEA'#13#13'   www.dxatlas.com/profiler', mtInformation, [mbOk], 0);
end;








//------------------------------------------------------------------------------
//                               GUI
//------------------------------------------------------------------------------
procedure TMainForm.SetPanelLayout(AVert: boolean; ASplitPos: integer = 0);
begin
  if AVert
    then
      begin
      TabControl1.Align := alLeft;
      if ASplitPos = 0
        then TabControl1.Width := Min(400, Width div 2)
        else TabControl1.Width := ASplitPos;
      Splitter1.Align := alLeft;
      Splitter1.Left := MAXINT;
      end
    else
      begin
      TabControl1.Align := alTop;
      if ASplitPos = 0
        then TabControl1.Height := Height div 2
        else TabControl1.Height := ASplitPos;
      Splitter1.Align := alTop;
      Splitter1.Top := MAXINT;
      end;

  TreePanel.Align := alNone;
  TreePanel.Align := alClient;
end;


procedure TMainForm.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
begin
  VerticalSplitMNU.Checked := TabControl1.Align = alLeft;
  HorisontalSplitMNU.Checked := TabControl1.Align <> alLeft;

  ToolButton1.Enabled := State = psIdle;
  ToolButton3.Enabled := (State = psIdle) and FileExists(Settings.AppName);
  ToolButton4.Enabled := State in [psWaiting, psSampling, psPaused];
  ToolButton5.Enabled := State in [psWaiting, psSampling, psPaused];

  ToolButton4.Down := State = psPaused;
end;


procedure TMainForm.SetState(const Value: TProfilerState);
const
  StateText: array[TProfilerState] of string =
    (' Idle', ' Waiting...', ' Sampling...', ' Paused');
begin
  FState := Value;
  StatusBar1.SimpleText := StateText[State];
  StatusBar1.Update;
end;


procedure TMainForm.ComboBox1Change(Sender: TObject);
begin
  ViewFrame1.Clear;
  TreeFrame1.PopulateTree;
end;


procedure TMainForm.TabControl1Change(Sender: TObject);
begin
  Panel4.Visible := TabControl1.TabIndex = 0;
  if Panel4.Visible
    then Panel5.Top := Panel4.Top + Panel4.Height
    else Panel5.Top := Panel4.Top;

  ViewFrame1.Clear;
  TreeFrame1.PopulateTree;
end;


procedure TMainForm.TreePanelResize(Sender: TObject);
begin
  TreeFrame1.AdjustColWidth;
end;


procedure TMainForm.FormShow(Sender: TObject);
begin
  TreeFrame1.AdjustColWidth;
end;










//------------------------------------------------------------------------------
//                             start/stop app
//------------------------------------------------------------------------------
function TMainForm.StartApp: boolean;
begin
  Result := false;
  Perf.Clear;
  TreeFrame1.Clear;
  ViewFrame1.Clear;

  ViewFrame1.Finder.SetProject(Settings.AppName);
  App.ExePath := ViewFrame1.Finder.BinFile;
  if not FileExists(App.ExePath) then
    begin
    MessageDlg('File does not exist: ' + App.ExePath, mtError, [mbOK], 0);
    Exit;
    end;

  if not App.RunAsync(true) then Exit;

  Perf.Clear;
  Perf.Pid := App.ProcessInfo.dwProcessId;
  Perf.ProcessName := ExtractFileName(App.ExePath);
  Perf.Modules.ListDrivers;

  ViewFrame1.Finder.BorToolIndex := Settings.BorTool;

  Spl.BufSize := Settings.BufSize;
  Spl.SamplingRate := Settings.SamplingRate;
  Spl.EventBased := Settings.EventBased;
  Spl.EventId := Settings.EventId;
  Spl.EventCount := Settings.EventCount;
  Spl.Open;
  TimeLeft := Settings.SampingSeconds / 86400;

  //start paused: sampling will start on DebugString command
  if Settings.StartPaused then State := psPaused

  //start delayed: set up the delay timer
  else if Settings.StartDelayed then
    begin
    Timer1.Interval := Settings.WarmUpSeconds * 1000;
    Timer1.Enabled := true;
    State := psWaiting;
    end

  //start now: begin sampling
  else StartSampling;

  Result := true;
end;


procedure TMainForm.StopApp;
begin
  App.Stop;
end;


procedure TMainForm.ShowData;
var
  i: integer;
  Lst: TStringList;
begin
  //list threads
  Lst := TStringList.Create;
  try
    for i:=0 to Perf.Threads.Count-1 do
      Lst.AddObject(TThreadItem(Perf.Threads[i]).Name, Ptr(i));
    Lst.Sort;
    Lst.InsertObject(0, 'ALL THREADS', Ptr(-1));
    ComboBox1.Items.Assign(Lst);

    SetLength(Perf.ThreadNumbers, Lst.Count);
    for i:=0 to Lst.Count-1 do Perf.ThreadNumbers[i] := Integer(Lst.Objects[i]);
  finally
    Lst.Free;
  end;
  ComboBox1.ItemIndex := 0;

  //show in the tree
  TabControl1.TabIndex := 0; 
  TabControl1Change(nil);
  TreeFrame1.PopulateTree;
end;






//------------------------------------------------------------------------------
//                        start/stop  sampling
//------------------------------------------------------------------------------
procedure TMainForm.StartSampling;
begin
  TimeLeft := Settings.SampingSeconds / 86400;
  ResumeSampling;
end;

procedure TMainForm.ResumeSampling;
begin
  Timer1.Enabled := false;
  Spl.Start;

  //set up stop timer
  if Settings.AutoStop then
    begin
    StartTime := Now;
    Timer1.Interval := Round(TimeLeft * 86400000);
    Timer1.Enabled := true;
    end;

  State := psSampling;
end;


procedure TMainForm.PauseSampling;
begin
  Timer1.Enabled := false;
  Spl.Stop;

  if State = psSampling
    then TimeLeft := TimeLeft - (Now - StartTime)
    else TimeLeft := Settings.SampingSeconds / 86400;


  State := psPaused;
end;


procedure TMainForm.StopSampling;
begin
  PauseSampling;
  State := psIdle;

  Spl.Close;
  PostMessage(Handle, WM_SAMPLING_STOPPED, 0, 0);
end;






//------------------------------------------------------------------------------
//                               events
//------------------------------------------------------------------------------
procedure TMainForm.DebugEvent(Sender: TObject);
var
  ThrId: DWord;
begin
  with App.DebugEvent do
    case dwDebugEventCode of
      CREATE_PROCESS_DEBUG_EVENT:
        begin
        Perf.Modules.AddModule(CreateProcessInfo.lpBaseOfImage, CreateProcessInfo.hFile, true);
        Perf.Threads.AddThread(dwThreadId, true);
        end;

      CREATE_THREAD_DEBUG_EVENT:
        Perf.Threads.AddThread(dwThreadId);

      LOAD_DLL_DEBUG_EVENT:
        Perf.Modules.AddModule(LoadDll.lpBaseOfDll, LoadDll.hFile);

     OUTPUT_DEBUG_STRING_EVENT:
       if Copy(App.DebugString, 1, 7) = 'THREAD ' then
         begin
         ThrId := StrToInt('$' + Copy(App.DebugString, 8, 8));
         Perf.Threads.SetThreadName(ThrId, Trim(Copy(App.DebugString, 17, MAXINT)));
         end

       else if (App.DebugString = 'SAMPLING ON') and Settings.StartPaused and (State = psPaused) then
         StartSampling;
     end;
end;


procedure TMainForm.TerminateEvent(Sender: TObject);
begin
  StopSampling;
end;


procedure TMainForm.Timer1Timer(Sender: TObject);
begin
  case State of
    psWaiting: StartSampling;
    psSampling:  StopApp;
    end;
end;

   
procedure TMainForm.ModuleEvent(Sender: TObject; ModuleName: string; Percent: Single);
begin
  if Settings.CanDownload then
    begin
    DlProgressDialog.ShowProgress(ModuleName, Percent);
    Application.ProcessMessages;
    end;
end;

procedure TMainForm.WmSamplingStopped(var Msg: TMessage);
var
  S: string;
begin
  Perf.Modules.GetMissingData;
  Perf.LoadSamples(Spl.RawFileName);

  if Perf.BufferFull then
    begin
    S := Format('%d samples collected, buffer full', [Perf.ProcessHits]);
    MessageDlg(S, mtWarning, [mbOK], 0);
    end;

  ForceDirectories(MainForm.Settings.SymbolsPath);
  SetSymbolsPath(Settings.SymbolsPath, Settings.CanDownload);
  Perf.LoadDebugInfo;

  ShowData;

  DlProgressDialog.Close;
end;






//------------------------------------------------------------------------------
//                                   ini
//------------------------------------------------------------------------------
procedure TMainForm.SaveToIni;
var
  WinPlacement: TWindowPlacement;
  R: PRect;
begin
  with TIniFile.Create(ChangeFileExt(ParamStr(0), '.ini')) do
    try
      //save window rect - works correctly even if window maximized
      WinPlacement.length := SizeOf(TWindowPlacement);
      R := @WinPlacement.rcNormalPosition;
      if GetWindowPlacement(MainForm.Handle, @WinPlacement) then
        begin
        WriteInteger('Window', 'Left', R.Left);
        WriteInteger('Window', 'Top', R.Top);
        WriteInteger('Window', 'Right', R.Right);
        WriteInteger('Window', 'Bottom', R.Bottom);
        end;

      //splitter
      WriteBool('Window', 'VertSplit', TabControl1.Align = alLeft);
      if TabControl1.Align = alLeft
        then WriteInteger('Window', 'SplitPos', TabControl1.Width)
        else WriteInteger('Window', 'SplitPos', TabControl1.Height);
    finally
      Free;
    end;
end;


procedure TMainForm.LoadFromIni;
var
  WinPlacement: TWindowPlacement;
  R: PRect;
begin
  with TIniFile.Create(ChangeFileExt(ParamStr(0), '.ini')) do
    try
      WinPlacement.length := SizeOf(TWindowPlacement);
      R := @WinPlacement.rcNormalPosition;
      if GetWindowPlacement(MainForm.Handle, @WinPlacement) then
        begin
        WinPlacement.showCmd := SW_HIDE;
        //set default window rect
        SystemParametersInfo(SPI_GETWORKAREA, 0, R, 0);
        InflateRect(R^, -50, -50);
        //read saved window rect
        R.Left := ReadInteger('Window', 'Left', R.Left);
        R.Top := ReadInteger('Window', 'Top', R.Top);
        R.Right := ReadInteger('Window', 'Right', R.Right);
        R.Bottom := ReadInteger('Window', 'Bottom', R.Bottom);
        //update window settings
        SetWindowPlacement(MainForm.Handle, @WinPlacement);
        end;

      //splitter
      if ReadBool('Window', 'VertSplit', true)
        then SetPanelLayout(true, ReadInteger('Window', 'SplitPos', 0))
        else SetPanelLayout(false, ReadInteger('Window', 'SplitPos', 0));
    finally
      Free;
    end;
end;



end.

