{
/*
 * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
 * Julian Hall. All rights reserved. The software is
 * redistributable under the license given in the file "LICENSE"
 * distributed in the NASM archive.
 */
}

//Declarations ported to Object Pascal by Alex Shovkoplyas, VE3NEA

unit NdisAsm;

interface


uses
  Windows;


const
  NdisAsmDll = 'Ndisasm.dll';

  INSN_MAX = 32;
  MAXPREFIX = 4;
  MAX_OPERANDS = 5;
  BITS32 = 4;

  OPTYPE_MASK	= $0000f000;
  REGISTER	  = $00001000;
  IMMEDIATE  	= $00002000;
  MEMORY	    = $0000c000;
  REGMEM	    = $00008000;

  MEM_OFFS    = $0001c000;
  IP_REL		  = $0002c000;

  I_CALL = 0038;
  I_JMP = 0438;
  I_Jcc = 1400;
  I_PUSH = 0834;

type
  TOperand = record
    OpType: integer;                            // type of operand
    disp_size: integer;                         // 0 means default; 16; 32; 64
    {enum}basereg, indexreg: DWord;             // address registers
    scale: integer;			                        // index scale
    hintbase: integer;
    {enum}hinttype: DWord;                      // hint as to real base register
    segment: integer;                           // immediate segment, if needed
    offset: integer;                            // any immediate number
    wrt: integer;                               // segment base it's relative to
    eaflags: integer;                           // special EA flags
    opflags: integer;                           // see OPFLAG_* defines below
    end;


  PInstructionDetails = ^TInstructionDetails;
  TInstructionDetails = record
    sLabel: PChar;                              // the label defined, or NULL
    {enum} prefixes: array[0..MAXPREFIX-1] of DWord; // instruction prefixes, if any
    {enum} opcode: DWord;                       // the opcode - not just the string
    {enum} condition: DWord;                    // the condition code, if Jcc/SETcc
    operands: integer;                          // how many operands? 0-3 * (more if db et al)
    addr_size: integer;		                      // address size
    oprs: array[0..MAX_OPERANDS-1] of TOperand; // the operands, defined as above
    eops: Pointer;                          // extended operands
    eops_float: integer;                        // true if DD and floating
    times: integer;                             // repeat count (TIMES prefix)
    forw_ref: LongBool;                             // is there a forward reference?
    rex: integer;			                          // Special REX Prefix
    drexdst: integer;		                        // Destination register for DREX/VEX suffix
    vex_m: integer;			                        // M register for VEX prefix
    vex_wlp: integer;		                        // W, P and L information for VEX prefix
    end;


  PInstructionTemplate = ^TInstructionTemplate;
  TInstructionTemplate = record
    {enum} opcode: DWord;                       // the token, passed from "parser.c"
    operands: integer;                          // number of operands
    opd: array [0..MAX_OPERANDS-1] of DWord;    // bit flags for operand types
    code: PByte;                                // the code it assembles to
    flags: DWord;                               // some flags
    end;


  PInstructionInfo = ^TInstructionInfo;
  TInstructionInfo = record
    Templ: PInstructionTemplate;
    Instr: TInstructionDetails;
    end;


function DisassembleInstruction(Instr: PByte; Offset: DWord): boolean;


var
  DisAsmLength: integer;
  DisAsmString: string;
  DisAsmInfo: TInstructionInfo;


  
implementation


function Disassemble(Data: PChar; Output: PChar; OutBufSize, SegSize, Offset:
  integer; AutoSync: integer; Prefer: DWord; Info: PInstructionInfo): LongInt; stdcall; external NdisAsmDll;



function DisassembleInstruction(Instr: PByte; Offset: DWord): boolean;
begin
  SetLength(DisAsmString, 256);
  DisAsmLength := Disassemble(PChar(Instr), @DisAsmString[1],
    Length(DisAsmString), 32, Integer(Offset), 0, 0, @DisAsmInfo);
    
  DisAsmString := PChar(@DisAsmString[1]);
  Result := DisAsmLength > 0;
end;



end.

