//------------------------------------------------------------------------------
//The contents of this file are subject to the Mozilla Public License
//Version 1.1 (the "License"); you may not use this file except in compliance
//with the License. You may obtain a copy of the License at
//http://www.mozilla.org/MPL/ Software distributed under the License is
//distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
//or implied. See the License for the specific language governing rights and
//limitations under the License.
//
//The Original Code is PathUtil.pas.
//
//The Initial Developer of the Original Code is Alex Shovkoplyas, VE3NEA.
//Portions created by Alex Shovkoplyas are
//Copyright (C) 2008 Alex Shovkoplyas. All Rights Reserved.
//------------------------------------------------------------------------------
unit PathUtil;

interface

uses
  Windows, SysUtils, Classes, PsAPI;

  
var
  WinDir, SysDir, DrvDir: TFileName;

function FixPath(Path: TFileName): TFileName;
function FileNameFromHandle(FileHandle: THandle): TFileName;




implementation


procedure GetSysDirectories;
begin
  SetLength(WinDir, MAX_PATH);
  GetWindowsDirectory(@WinDir[1], MAX_PATH);
  WinDir := PChar(@WinDir[1]);

  SetLength(SysDir, MAX_PATH);
  GetSystemDirectory(@SysDir[1], MAX_PATH);
  SysDir := PChar(@SysDir[1]);

  DrvDir := SysDir + '\drivers';
end;


function FileNameFromHandle(FileHandle: THandle): TFileName;
var
  FileSizeHi: DWord;
  Hm: THandle;
  P: Pointer;
begin
  Result := '';
  if (GetFileSize(FileHandle, @FileSizeHi) = 0) and (FileSizeHi = 0) then Exit;

  Hm := CreateFileMapping(FileHandle, nil, PAGE_READONLY, 0, 1, nil);
  if Hm = 0 then Exit;

  try
    P := MapViewOfFile(Hm, FILE_MAP_READ, 0, 0, 1);
    if P = nil then Exit;
    try
      SetLength(Result, MAX_PATH);
      SetLength(Result, GetMappedFileName(GetCurrentProcess, P, @Result[1], MAX_PATH));
    finally
      UnmapViewOfFile(P);
    end;
  finally
    CloseHandle(Hm);
  end;
end;


function MapDeviceToDrive(Path: TFileName): TFileName;
var
  Drives, LogDrive, Drive: string;
  Len, i: integer;
begin
  Result := Path;

  //list logical drive names
  SetLength(Drives, MAX_PATH);
  Len := GetLogicalDriveStrings(MAX_PATH-1, @Drives[1]);
  if Len = 0 then Exit;
  SetLength(Drives, Len);
  for i:=1 to Length(Drives) do if Drives[i] = #0 then Drives[i] := #13;

  //replace device name with logical drive name
  with TStringList.Create do
    try
      Text := Drives;
      for i:=0 to Count-1 do
        begin
        LogDrive := Strings[i][1] + ':';
        SetLength(Drive, MAX_PATH);
        SetLength(Drive, QueryDosDevice(PChar(LogDrive), @Drive[1], MAX_PATH));
        Drive := Trim(Drive);
        Len := Length(Drive);
        if Drive <> Copy(Path, 1, Len) then Continue;
        Result := LogDrive + Copy(Path, Len+1, MAXINT);
        Break;
        end;
    finally
      Free;
    end;
end;


function FixPath(Path: TFileName): TFileName;
begin
  Path := Trim(Path);

  if Copy(Path, 1, 4) = '\??\' then
    Result := Copy(Path, 5, MAXINT)

  else if Pos(Copy(SysDir, 3, MAXINT), Path) = 1 then
    Result := SysDir + Copy(Path, Length(SysDir)-1, MAXINT)

  else if Pos(Copy(WinDir, 3, MAXINT), Path) = 1 then
    Result := WinDir + Copy(Path, Length(WinDir)-1, MAXINT)

  else if Pos('\SystemRoot', Path) = 1 then
    Result := WinDir + Copy(Path, 12, MAXINT)

  else if Pos('\Device\', Path) = 1 then
    Result := MapDeviceToDrive(Path)

  else if Pos('\', Path) = 0 then
    begin
    Result := DrvDir + '\' + Path;
    if not FileExists(Result) then Result := Path;
    end

  else Result := Path;
end;



initialization
  GetSysDirectories;



end.

