{
/*
 * IBM Performance Inspector
 * Copyright (c) International Business Machines Corp., 1996 - 2008
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
}

//Parts of perfutil.h ported to Object Pascal by Alex Shovkoplyas, VE3NEA


unit PerfUtil;

interface

uses
  Windows, SysUtils, Registry, PathUtil;

const
  HOOK_MAX_INTS = 128;
  HOOK_MAX_STRS = 16;
  HOOK_MAX_STR_LEN = 2048;
  MAX_CPUS = 32;
  TRACE_MAX_METRICS = 8;

  COLLECTION_MODE_NORMAL = 2;
  PI_TRACE_BUF_DEFAULT = $300000; //3 Mb
  IBMPERF_TPROF_TICK_RATE = 128;
  PRIVILEGE_LEVEL_ALL = 3;
  ALL_CPUS = -1;
  EVENT_INVALID_ID = 0;

  //hook majors
  SFT_HOOK_TSCHANGE_MAJOR = $08;
  SFT_HOOK_TPROF_MAJOR = $10;
  SYS_INFO_MAJOR = $11;
  SFT_HOOK_MTE_MAJOR = $19;

  //MTE hook minors
  MTE_MINOR                  = $01;
  PID_NAME_MINOR             = $34;
  SFT_HOOK_3264BIT_MINOR     = $35;
  SFT_HOOK_CREATE_PID_MINOR  = $64;
  SFT_HOOK_KILL_PID_MINOR    = $65;
  SFT_HOOK_CREATE_THR_MINOR  = $66;
  SFT_HOOK_KILL_THR_MINOR    = $67;

  NRM2_SIGNATURE = $324D524E;
  RAWFMT_HOOK_LEN_MASK  = $FFFC;


type
  PTraceHook = ^TTraceHook;
  TTraceHook = record
    raw: PByte;
    hook_type: Word;
    hook_len: Word;
    cpu_no: Word;
    major: Word;
    minor: DWord;
    tlo: DWord;
    thi: DWord;
    file_offset: DWord;
    ints: Word;
    strings: Word;
    ival: array[0..HOOK_MAX_INTS-1] of DWord;
    slen: array[0..HOOK_MAX_STRS-1] of DWord;
    sval: array[0..HOOK_MAX_STRS-1, 0..HOOK_MAX_STR_LEN-1] of Char;
    end;


  PNrmFileInfo = ^TNrmFileInfo;
  TNrmFileInfo = record
    num_cpus: integer;
    buffer_size: DWord;
    data_size: array[0..MAX_CPUS-1] of DWord;
    hook_cnt: array[0..MAX_CPUS-1] of integer;
    buffer_full: array[0..MAX_CPUS-1] of integer;
    drift: array[0..MAX_CPUS-1] of Int64;
    cpu_speed_mhz: DWord;
    ascii: integer;
    cr_value: integer;
    little_endian: integer;
    type01_no_ts: integer;
    interval_id: integer;
    num_metrics: integer;
    metrics: array[0..TRACE_MAX_METRICS-1] of integer;
    os_32bits: integer;
    cpu_architecture: integer;
    os: array[0..7] of Char;
  end;


  PPerfCtrEventInfo = ^TPerfCtrEventInfo;
  TPerfCtrEventInfo = record
     name: PChar;                    // Event name         
     description: PChar;             // Event description (about one sentence)         
     hdw_event_name: PChar;          // Event name used in hardware documents         
     um_em: PChar;                   // Unit Mask (P6) or Event Mask (P4) value         
     event_id: integer;              // Event Id         
     num_ctrs: integer;              // Number of counters where event can be counted         
     ctrs: array[0..5] of integer;   // Actual counter numbers
     end;



  
var
  IsDeviceDriverInstalled: function: integer; stdcall;
  IsLimitedFunctionMode: function: integer; stdcall;
  IsPerfCtrAccessSupported: function: integer; stdcall;
  IsPerfCtrEventAccessSupported: function: integer; stdcall;
  AreInterruptHooksSupported: function: integer; stdcall;
  IsDispatchHookSupported: function: integer; stdcall;

  IsTraceOn: function: integer; stdcall;
  TraceResume: function: integer; stdcall;
  TraceSuspend: function: integer; stdcall;

  //stdcall?
  Nrm2OpenTraceFile: function(fnm: PChar): integer; cdecl;
  Nrm2CloseTraceFile: procedure; cdecl;
  Nrm2GetNextHook: function: PTraceHook; cdecl;
  Nrm2GetFileInfo: function(fi: PNrmFileInfo): integer; cdecl;
  Nrm2SetQuietMode: procedure(AOn: integer); cdecl;
  Nrm2SetRawDataMode: procedure(AOn: integer); cdecl;

  TraceInit: function(bufsize: integer): integer; cdecl;
  GetPerfCounterEventIdFromName: function(AName: PChar): integer; cdecl;
  GetPerfCounterNumberFromName: function: integer; cdecl;
  IsHyperThreadingEnabled: function: integer; cdecl;
  TraceSetTprofEvent: function(cpu, event, event_cnt, priv_level: integer): integer; cdecl;
  SetProfilerRate: function(timer_rate: DWord): integer; cdecl;
  TraceEnable: function(majorCode: integer): integer; cdecl;
  TraceOn: function: integer; cdecl;
  TraceOff: function: integer; cdecl;
  TraceDisable: function(majorCode: integer): integer; cdecl;
  TraceTerminate: function: integer; cdecl;
  TraceWriteBufferToFile: function(FileName: PChar): integer; cdecl;

  GetPerfCounterEventIdList: function(var event_id_list: PInteger): integer; cdecl;
  GetPerfCounterEventInfoFromId: function(event_id: integer): PPerfCtrEventInfo; cdecl;




function IsPerfUtilLoaded: boolean;



implementation

var
  DllHandle: THandle;

  
procedure LoadDll;
const
  PERF_UTIL_KEY = 'SYSTEM\CurrentControlSet\Services\PERFDD';
  PERF_UTIL_NAME = 'ImagePath';
  DLL_NAME = 'perfutil.dll';
var
  DllPath: TFileName;
begin
  //path to the dll
  DllPath := '';

  with TRegistry.Create do
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKeyReadOnly(PERF_UTIL_KEY) then
        DllPath := ReadString(PERF_UTIL_NAME);
    finally Free; end;

  DllPath := ExtractFilePath(FixPath(DllPath)) + DLL_NAME;

  //load dll
  DllHandle := LoadLibrary(PChar(DllPath));
  if DllHandle = 0 then Exit;

  //get proc adr
  @IsDeviceDriverInstalled := GetProcAddress(DllHandle, 'IsDeviceDriverInstalled');
  @IsLimitedFunctionMode := GetProcAddress(DllHandle, 'IsLimitedFunctionMode');
  @IsPerfCtrAccessSupported := GetProcAddress(DllHandle, 'IsPerfCtrAccessSupported');
  @IsPerfCtrEventAccessSupported := GetProcAddress(DllHandle, 'IsPerfCtrEventAccessSupported');
  @AreInterruptHooksSupported := GetProcAddress(DllHandle, 'AreInterruptHooksSupported');
  @IsDispatchHookSupported := GetProcAddress(DllHandle, 'IsDispatchHookSupported');
  @IsTraceOn := GetProcAddress(DllHandle, 'IsTraceOn');
  @TraceResume := GetProcAddress(DllHandle, 'TraceResume');
  @TraceSuspend := GetProcAddress(DllHandle, 'TraceSuspend');

  @Nrm2OpenTraceFile := GetProcAddress(DllHandle, 'Nrm2OpenTraceFile');
  @Nrm2CloseTraceFile := GetProcAddress(DllHandle, 'Nrm2CloseTraceFile');
  @Nrm2GetNextHook := GetProcAddress(DllHandle, 'Nrm2GetNextHook');
  @Nrm2GetFileInfo := GetProcAddress(DllHandle, 'Nrm2GetFileInfo');
  @Nrm2SetQuietMode := GetProcAddress(DllHandle, 'Nrm2SetQuietMode');
  @Nrm2SetRawDataMode := GetProcAddress(DllHandle, 'Nrm2SetRawDataMode');
  @TraceInit := GetProcAddress(DllHandle, 'TraceInit');
  @GetPerfCounterEventIdFromName := GetProcAddress(DllHandle, 'GetPerfCounterEventIdFromName');
  @GetPerfCounterNumberFromName := GetProcAddress(DllHandle, 'GetPerfCounterNumberFromName');
  @IsHyperThreadingEnabled := GetProcAddress(DllHandle, 'IsHyperThreadingEnabled');
  @TraceSetTprofEvent := GetProcAddress(DllHandle, 'TraceSetTprofEvent');
  @SetProfilerRate := GetProcAddress(DllHandle, 'SetProfilerRate');
  @TraceEnable := GetProcAddress(DllHandle, 'TraceEnable');
  @TraceOn := GetProcAddress(DllHandle, 'TraceOn');
  @TraceOff := GetProcAddress(DllHandle, 'TraceOff');
  @TraceDisable := GetProcAddress(DllHandle, 'TraceDisable');
  @TraceTerminate := GetProcAddress(DllHandle, 'TraceTerminate');
  @TraceWriteBufferToFile := GetProcAddress(DllHandle, 'TraceWriteBufferToFile');

  @GetPerfCounterEventIdList := GetProcAddress(DllHandle, 'GetPerfCounterEventIdList');
  @GetPerfCounterEventInfoFromId := GetProcAddress(DllHandle, 'GetPerfCounterEventInfoFromId');


  if (@IsDeviceDriverInstalled = nil) or
     (@IsLimitedFunctionMode = nil) or
     (@IsPerfCtrAccessSupported = nil) or
     (@IsPerfCtrEventAccessSupported = nil) or
     (@AreInterruptHooksSupported = nil) or
     (@IsDispatchHookSupported = nil) or
     (@IsTraceOn = nil) or
     (@TraceResume = nil) or
     (@TraceSuspend = nil) or

     (@Nrm2OpenTraceFile = nil) or
     (@Nrm2CloseTraceFile = nil) or
     (@Nrm2GetNextHook = nil) or
     (@Nrm2GetFileInfo = nil) or
     (@Nrm2SetQuietMode = nil) or
     (@Nrm2SetRawDataMode = nil) or
     (@TraceInit = nil) or
     (@GetPerfCounterEventIdFromName = nil) or
     (@GetPerfCounterNumberFromName = nil) or
     (@IsHyperThreadingEnabled = nil) or
     (@TraceSetTprofEvent = nil) or
     (@SetProfilerRate = nil) or
     (@TraceEnable = nil) or
     (@TraceOn = nil) or
     (@TraceOff = nil) or
     (@TraceDisable = nil) or
     (@TraceTerminate = nil) or
     (@TraceWriteBufferToFile = nil) or
     (@GetPerfCounterEventIdList = nil) or
     (@GetPerfCounterEventInfoFromId = nil)

      or (IsDeviceDriverInstalled = 0) or (IsLimitedFunctionMode <> 0)
    then
      begin
      FreeLibrary(DllHandle);
      DllHandle := 0;
      end;
end;


function IsPerfUtilLoaded: boolean;
begin
  Result := DllHandle <> 0;
end;



initialization
  LoadDll;


finalization
  if DllHandle <> 0 then FreeLibrary(DllHandle);



end.

