//------------------------------------------------------------------------------
//The contents of this file are subject to the Mozilla Public License
//Version 1.1 (the "License"); you may not use this file except in compliance
//with the License. You may obtain a copy of the License at
//http://www.mozilla.org/MPL/ Software distributed under the License is
//distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
//or implied. See the License for the specific language governing rights and
//limitations under the License.
//
//The Original Code is SettDlg.pas.
//
//The Initial Developer of the Original Code is Alex Shovkoplyas, VE3NEA.
//Portions created by Alex Shovkoplyas are
//Copyright (C) 2008 Alex Shovkoplyas. All Rights Reserved.
//------------------------------------------------------------------------------
unit SettDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, Spin, Math, FileCtrl;

type
  TSettingsDialog = class(TForm)
    Panel1: TPanel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Button1: TButton;
    Button2: TButton;
    Edit1: TEdit;
    Label1: TLabel;
    SpeedButton1: TSpeedButton;
    GroupBox1: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    SpinEdit1: TSpinEdit;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    SpinEdit2: TSpinEdit;
    RadioButton5: TRadioButton;
    Label3: TLabel;
    OpenDialog1: TOpenDialog;
    TabSheet3: TTabSheet;
    Label4: TLabel;
    SpeedButton2: TSpeedButton;
    Edit2: TEdit;
    CheckBox1: TCheckBox;
    Label7: TLabel;
    Label8: TLabel;
    SpinEdit3: TSpinEdit;
    Label9: TLabel;
    ComboBox2: TComboBox;
    RadioButton6: TRadioButton;
    RadioButton7: TRadioButton;
    GroupBox3: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    ComboBox1: TComboBox;
    GroupBox4: TGroupBox;
    Label11: TLabel;
    ComboBox3: TComboBox;
    Label10: TLabel;
    SpinEdit4: TSpinEdit;
    Label12: TLabel;
    procedure FormShow(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure RadioButtonClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure ComboBox3Change(Sender: TObject);
    procedure RadioButton6Click(Sender: TObject);
  private
    procedure FromDialog;
    procedure ToDialog;
    function GetEventId: integer;
  end;


var
  SettingsDialog: TSettingsDialog;



implementation

{$R *.DFM}

uses
  Main;


procedure TSettingsDialog.FormShow(Sender: TObject);
begin
  ToDialog;
end;


procedure TSettingsDialog.Button1Click(Sender: TObject);
begin
  FromDialog;
end;


procedure TSettingsDialog.ToDialog;
var
  Idx: integer;
begin
  Edit1.Text := MainForm.Settings.AppName;

  RadioButton1.Checked := not (MainForm.Settings.StartDelayed or MainForm.Settings.StartPaused);
  RadioButton2.Checked := MainForm.Settings.StartDelayed;
  RadioButton5.Checked := MainForm.Settings.StartPaused;

  RadioButton3.Checked := not MainForm.Settings.AutoStop;
  RadioButton4.Checked := MainForm.Settings.AutoStop;

  SpinEdit1.Value := MainForm.Settings.WarmUpSeconds;
  SpinEdit2.Value := MainForm.Settings.SampingSeconds;

  Edit2.Text := MainForm.Settings.SymbolsPath;
  CheckBox1.Checked := MainForm.Settings.CanDownload;

  SpinEdit3.Value := MainForm.Settings.BufSize shr 20;

  Idx := Round(Log2(MainForm.Settings.SamplingRate)) - 5;
  ComboBox1.ItemIndex := Max(0, Min(ComboBox1.Items.Count, Idx));

  ComboBox2.Items.Text := MainForm.ViewFrame1.Finder.GetToolNames;
  ComboBox2.Items.Insert(0, '<None>');
  ComboBox2.ItemIndex := MainForm.Settings.BorTool + 1;

  RadioButton7.Checked := MainForm.Settings.EventBased;
  RadioButton6.Checked := not MainForm.Settings.EventBased;

  SpinEdit4.Value := MainForm.Settings.EventCount;

  ComboBox3.Items.Assign(MainForm.Spl.EventNames);
  if ComboBox3.Items.Count > 0 then ComboBox3.ItemIndex := 0;
  for Idx:=0 to ComboBox3.Items.Count-1 do
    if Integer(MainForm.Spl.EventNames.Objects[Idx]) = MainForm.Settings.EventId
      then begin ComboBox3.ItemIndex := Idx; Break; end;
  ComboBox3Change(nil);
end;


procedure TSettingsDialog.FromDialog;
begin
  MainForm.Settings.AppName := Edit1.Text;

  MainForm.Settings.StartDelayed := RadioButton2.Checked;
  MainForm.Settings.StartPaused  := RadioButton5.Checked;

  MainForm.Settings.AutoStop := RadioButton4.Checked;

  MainForm.Settings.WarmUpSeconds := SpinEdit1.Value;
  MainForm.Settings.SampingSeconds := SpinEdit2.Value;

  MainForm.Settings.SymbolsPath := Edit2.Text;
  MainForm.Settings.CanDownload := CheckBox1.Checked;

  MainForm.Settings.BufSize := SpinEdit3.Value shl 20;
  MainForm.Settings.SamplingRate := 32 shl (ComboBox1.ItemIndex);

  MainForm.Settings.BorTool := ComboBox2.ItemIndex - 1;

  MainForm.Settings.EventCount := SpinEdit4.Value;

  MainForm.Settings.EventBased := RadioButton7.Checked;
  MainForm.Settings.EventId := GetEventId;
end;


procedure TSettingsDialog.RadioButtonClick(Sender: TObject);
begin
  SpinEdit1.Enabled := RadioButton2.Checked;
  SpinEdit2.Enabled := RadioButton4.Checked;
end;


procedure TSettingsDialog.SpeedButton1Click(Sender: TObject);
begin
  if not OpenDialog1.Execute then Exit;
  Edit1.Text := OpenDialog1.FileName;
end;



procedure TSettingsDialog.SpeedButton2Click(Sender: TObject);
var
  Dir: string;
begin
  Dir := Edit2.Text;
  if SelectDirectory('Select symbol storage folder', '', Dir)
    then Edit2.Text := Dir;
end;


procedure TSettingsDialog.ComboBox3Change(Sender: TObject);
var
  Id: integer;
begin
  Id := GetEventId;

  if Id < 0
    then Label10.Caption := ''
    else Label10.Caption := MainForm.Spl.GetEventDescription(Id);
end;


function TSettingsDialog.GetEventId: integer;
begin
  if Combobox3.ItemIndex < 0
    then Result := -1
    else Result := Integer(MainForm.Spl.EventNames.Objects[Combobox3.ItemIndex]);
end;


procedure TSettingsDialog.RadioButton6Click(Sender: TObject);
begin
  ComboBox1.Enabled := RadioButton6.Checked;
  ComboBox3.Enabled := RadioButton7.Checked;
  SpinEdit4.Enabled := RadioButton7.Checked;
end;



end.

