//------------------------------------------------------------------------------
//The contents of this file are subject to the Mozilla Public License
//Version 1.1 (the "License"); you may not use this file except in compliance
//with the License. You may obtain a copy of the License at
//http://www.mozilla.org/MPL/ Software distributed under the License is
//distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
//or implied. See the License for the specific language governing rights and
//limitations under the License.
//
//The Original Code is TreeFrm.pas.
//
//The Initial Developer of the Original Code is Alex Shovkoplyas, VE3NEA.
//Portions created by Alex Shovkoplyas are
//Copyright (C) 2008 Alex Shovkoplyas. All Rights Reserved.
//------------------------------------------------------------------------------
unit TreeFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, ThrLst, Math, ImgList, HitCnts, ModulInf, Menus;

type
  TTotalEntry = record
    Caption: string;
    ImageIndex: integer;
    Count: integer;
    end;


  TTreeFrame = class(TFrame)
    Tv: TVirtualStringTree;
    ImageList1: TImageList;
    procedure TvGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer; TextType: TVSTTextType; var Text: WideString);
    procedure TvGetImageIndex(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Kind: TVTImageKind; Column: Integer; var Ghosted: Boolean;
      var Index: Integer);
    procedure TvHeaderClick(Sender: TVTHeader; Column: Integer);
    procedure TvCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: Integer; var Result: Integer);
    procedure TvClick(Sender: TObject);
    procedure TvGetHint(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer; TextType: TVSTTextType; var Text: WideString);
    procedure TvAfterItemErase(Sender: TBaseVirtualTree; Canvas: TCanvas;
      Node: PVirtualNode; ItemRect: TRect);
    procedure TvFocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer);
  private
    TotEntries: array of TTotalEntry;
    TotHits: integer;
    Tab: integer;
    Thr: integer;

    procedure ShowModules;
    procedure ShowTotals;
    procedure AddChildren(AParentNode: PVirtualNode; ACounter: TCustomHitCounter);
    function GetNodeCaption(Node: PVirtualNode): string;
    function GetHits(Counter: TCustomHitCounter): integer; overload;
    function GetHits(Node: PVirtualNode): integer; overload;
    function IsCountBlank(Node: PVirtualNode): boolean;
    procedure DataFindCallback(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
  public

    procedure Clear;
    procedure PopulateTree;
    procedure ExpandFirstLevel;
    procedure AdjustColWidth;
  end;

  

implementation

{$R *.DFM}

uses
  Main;

//------------------------------------------------------------------------------
//                                  public
//------------------------------------------------------------------------------
{ TTreeFrame }

procedure TTreeFrame.Clear;
begin
  Tv.Clear;
  Tv.Update;
end;


procedure TTreeFrame.PopulateTree;
var
  OldFocusedData: TCustomHitCounter;
begin
  if MainForm.Perf.IsEmpty then begin Clear; Exit; end;

  Tab := MainForm.TabControl1.TabIndex;
  TotHits := Max(1, MainForm.Perf.ProcessHits);

  if Tv.FocusedNode = nil
    then OldFocusedData := nil
    else OldFocusedData := TCustomHitCounter(Tv.GetNodeData(Tv.FocusedNode)^);

  if MainForm.ComboBox1.ItemIndex = -1
    then Thr := -1
    else Thr := MainForm.Perf.ThreadNumbers[MainForm.ComboBox1.ItemIndex];

  TotEntries := nil;

  Tv.BeginUpdate;
  try
    Tv.TreeOptions.AutoOptions := Tv.TreeOptions.AutoOptions - [toAutoSort];
    Tv.Clear;
    case Tab of
      0: ShowModules;
      1: ShowTotals;
      end;
    AdjustColWidth;
    Tv.Header.SortColumn := 1;
    Tv.TreeOptions.AutoOptions := Tv.TreeOptions.AutoOptions + [toAutoSort];
  finally
    Tv.EndUpdate;
  end;


  //focus
  MainForm.ActiveControl := Tv;
  if (Tab = 0) and (Tv.RootNodeCount > 0)
    then
      begin
      if OldFocusedData <> nil then
        Tv.FocusedNode := Tv.IterateSubtree(nil, DataFindCallback, OldFocusedData, [], false, false);

      if Tv.FocusedNode = nil then Tv.FocusedNode := Tv.RootNode.FirstChild;
      Tv.Selected[Tv.FocusedNode] := true;
      end
    else
      MainForm.ViewFrame1.ShowTotals;
end;


procedure TTreeFrame.DataFindCallback(Sender: TBaseVirtualTree; Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
begin
  Abort := Data = Pointer(Tv.GetNodeData(Node)^);
end;


procedure TTreeFrame.AdjustColWidth;
begin
  Tv.Header.Columns[0].Width := Tv.Width - 18 - Tv.Header.Columns[1].Width - Tv.Header.Columns[2].Width;
end;





   
//------------------------------------------------------------------------------
//                               modules
//------------------------------------------------------------------------------
procedure TTreeFrame.ShowModules;
var
  i: integer;
  Module: TModuleHits;
  Node: PVirtualNode;
begin
  for i:=0 to High(MainForm.Perf.HitsByModule) do
    begin
    Module := MainForm.Perf.HitsByModule[i];
    if GetHits(Module) = 0 then Continue;
    Node := Tv.AddChild(nil, Module);
    AddChildren(Node, Module);
    end;
end;


procedure TTreeFrame.AddChildren(AParentNode: PVirtualNode; ACounter: TCustomHitCounter);
var
  i: integer;
  Node: PVirtualNode;
begin
  //no info at any level, show addresses
  if ACounter.ChildCount = 0
    then Exit

   //no info at this level, go to next level
  else if (ACounter.ChildCount = 1) and (ACounter.Children[0].Name = STR_UNKNOWN)
    then AddChildren(AParentNode, ACounter.Children[0])

  //info available
  else
    for i:=0 to ACounter.ChildCount-1 do
      if GetHits(ACounter.Children[i]) > 0 then
        begin
        Node := Tv.AddChild(AParentNode, ACounter.Children[i]);
        AddChildren(Node, ACounter.Children[i]);
        end;
end;








//------------------------------------------------------------------------------
//                                 totals
//------------------------------------------------------------------------------
procedure TTreeFrame.ShowTotals;
var
  i, j: integer;
  Node: PVirtualNode;

  function AddItem(Txt: string; ImgIdx, ACount: integer; ParentNode: PVirtualNode): PVirtualNode;
  begin
    SetLength(TotEntries, Length(TotEntries)+1);
    with TotEntries[High(TotEntries)] do
      begin
      Caption := Txt;
      ImageIndex := ImgIdx;
      Count := ACount;
      end;
    Result := Tv.AddChild(ParentNode, Ptr(High(TotEntries)));
  end;

begin
  Tv.BeginUpdate;
  try
    Tv.Clear;
    if MainForm.Perf.IsEmpty then Exit;

    //CPU
    Node := AddItem('CPU''s', -1, MainForm.Perf.ProcessHits, nil);
    for i:=0 to High(MainForm.Perf.CpuHits) do
      AddItem(Format('CPU#%d', [i]), 0, MainForm.Perf.CpuHits[i], Node);

    //Modes
    Node := AddItem('Modes', -1, MainForm.Perf.ProcessHits, nil);
    Node := AddItem('User Mode', 10, MainForm.Perf.ModeHits[0], Node);
    for i:=0 to High(MainForm.Perf.ThrHits) do
      AddItem(TTHreadItem(MainForm.Perf.Threads[i]).Name, 3, MainForm.Perf.ModeThrHits[0,i], Node);
    Node := Node.Parent;
    Node := AddItem('Kernel Mode', 2, MainForm.Perf.ModeHits[1], Node);
    for i:=0 to High(MainForm.Perf.ThrHits) do
      AddItem(TTHreadItem(MainForm.Perf.Threads[i]).Name, 3, MainForm.Perf.ModeThrHits[1,i], Node);

    //Threads
    Node := AddItem('Threads', -1, MainForm.Perf.ProcessHits, nil);
    for i:=0 to High(MainForm.Perf.ThrHits) do
      begin
      Node := AddItem(TTHreadItem(MainForm.Perf.Threads[i]).Name, 3, MainForm.Perf.ThrHits[i], Node);
      for j:=0 to High(MainForm.Perf.CpuHits) do
        AddItem(Format('CPU#%d', [j]), 0, MainForm.Perf.CpuThrHits[j,i], Node);
      Node := Node.Parent;
      end;

    ExpandFirstLevel;
  finally
    Tv.EndUpdate;
  end;
end;






//------------------------------------------------------------------------------
//                                 get
//------------------------------------------------------------------------------
function TTreeFrame.GetNodeCaption(Node: PVirtualNode): string;
var
  Data: integer;
begin
  Data := PInteger(Tv.GetNodeData(Node))^;

  case Tab of
    0: Result := TCustomHitCounter(Data).Name;    //counter
    1: Result := TotEntries[Data].Caption;        //total
    end;
end;


function TTreeFrame.GetHits(Node: PVirtualNode): integer;
begin
  if Tab = 1
    then Result := TotEntries[PInteger(Tv.GetNodeData(Node))^].Count
    else Result := GetHits(TCustomHitCounter(Tv.GetNodeData(Node)^));
end;


function TTreeFrame.GetHits(Counter: TCustomHitCounter): integer;
begin
  if Counter = nil then Result := 0
  else if Thr < 0 then Result := Counter.Hits
  else Result := Counter.ThrHits[Thr];
end;











//------------------------------------------------------------------------------
//                                  tv events
//------------------------------------------------------------------------------
function TTreeFrame.IsCountBlank(Node: PVirtualNode): boolean;
begin
  Result := vsExpanded in Node.States;
  if not Result then Result := (Tab = 1) and (Node.Parent = Tv.RootNode);
end;

procedure TTreeFrame.TvGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
  var Text: WideString);
begin
  if Column = 0 then Text := GetNodeCaption(Node)
  else if IsCountBlank(Node) then Text := ''
  else if Column = 1 then Text := IntToStr(GetHits(Node))
  else Text := Format('%.1f%%', [100 * GetHits(Node) / TotHits]);
end;


procedure TTreeFrame.TvGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: Integer;
  var Ghosted: Boolean; var Index: Integer);
var
  Counter: TCustomHitCounter;
begin
  //images only in the 0-th column
  Index := -1;
  if Column > 0 then Exit;
  if Kind = ikOverlay then Exit;

  if Tab = 1
    then //totals
      Index := TotEntries[PInteger(Tv.GetNodeData(Node))^].ImageIndex
    else //modules
      begin
      Counter := TCustomHitCounter(Tv.GetNodeData(Node)^);
      if Counter is TAddrHits then Index := 4
      else if Counter is TLineHits then Index := 5
      else if Counter is TFuncHits then Index := 6
      else if Counter is TUnitHits then Index := 7
      else if Counter is TModuleHits then
        with Counter as TModuleHits do
          if Mode > 0 then Index := 2
          else if UpperCase(ExtractFileExt(Name)) = '.EXE' then Index := 9
          else Index := 10;
      end;
end;


procedure TTreeFrame.TvHeaderClick(Sender: TVTHeader; Column: Integer);
begin
  Tv.Header.SortColumn := Column;
end;


procedure TTreeFrame.TvCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: Integer; var Result: Integer);
begin
  if Column = 0
    then Result := CompareText(GetNodeCaption(Node2), GetNodeCaption(Node1))
    else Result := GetHits(Node1) - GetHits(Node2)
end;


procedure TTreeFrame.TvClick(Sender: TObject);
begin
  AdjustColWidth;
end;


procedure TTreeFrame.TvGetHint(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
  var Text: WideString);
var
  i: integer;
  Counter: TCustomHitCounter;
  Lst: TStringList;
  SpaceCount: integer;

  function Fmt(ACnt: integer; AThr: integer): string;
  var
    N: integer;
  begin
    Result := IntToStr(ACnt);
    N := (7-Length(Result)) * 2;
    SpaceCount := Min(SpaceCount, N);
    Result := StringOfChar(' ', N) + Result + '    ';
    Result := Result + TThreadItem(MainForm.Perf.Threads[AThr]).Name + #13;
  end;

begin
  Text := '';
  if (Tab > 0) or (Column = 0) then Exit;
  SpaceCount := MAXINT;

  Counter := TCustomHitCounter(Tv.GetNodeData(Node)^);
  for i:=0 to High(Counter.ThrHits) do
    if Counter.ThrHits[i] > 0 then
      Text := Text + Fmt(Counter.ThrHits[i], i);

  //remove leading spaces
  Lst := TStringList.Create;
  try
    Lst.Text := Text;
    for i:=0 to Lst.Count-1 do
      Lst[i] := Copy(Lst[i], SpaceCount+1, MAXINT);
    Text := Lst.Text;
  finally
    Lst.Free;
  end;
end;


procedure TTreeFrame.ExpandFirstLevel;
var
  Node: PVirtualNode;
begin
  Tv.FullCollapse;
  Node := Tv.RootNode.FirstChild;
  while Node <> nil do
    begin Tv.Expanded[Node] := true; Node := Node.NextSibling; end;
end;


procedure TTreeFrame.TvAfterItemErase(Sender: TBaseVirtualTree;
  Canvas: TCanvas; Node: PVirtualNode; ItemRect: TRect);
var
  W: integer;
begin
  Exit;

  if IsCountBlank(Node) then Exit;

  InflateRect(ItemRect, -Tv.Header.Columns[0].Width, -3);
  W := Tv.Header.Columns[1].Width + Tv.Header.Columns[2].Width;
  W := Round(W * GetHits(Node) / TotHits);
  if W < 2 then Exit;
  ItemRect.Right := ItemRect.Left + W + 1;

  Canvas.Brush.Color := $00FFD9D9;
  Canvas.FillRect(ItemRect);
  Canvas.Brush.Color := $00FF9393;
  Canvas.FrameRect(ItemRect);    // $00FFE8E8
end;



procedure TTreeFrame.TvFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer);
begin
  if Tab = 0 then
    MainForm.ViewFrame1.ShowItem(TCustomHitCounter(Tv.GetNodeData(Node)^));
end;

end.

