//------------------------------------------------------------------------------
//The contents of this file are subject to the Mozilla Public License
//Version 1.1 (the "License"); you may not use this file except in compliance
//with the License. You may obtain a copy of the License at
//http://www.mozilla.org/MPL/ Software distributed under the License is
//distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
//or implied. See the License for the specific language governing rights and
//limitations under the License.
//
//The Original Code is ViewFrm.pas.
//
//The Initial Developer of the Original Code is Alex Shovkoplyas, VE3NEA.
//Portions created by Alex Shovkoplyas are
//Copyright (C) 2008 Alex Shovkoplyas. All Rights Reserved.
//------------------------------------------------------------------------------
unit ViewFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, HitCnts, ComCtrls, ExtCtrls, SynEdit, FileCtrl, JclFileUtils,
  JclDateTime, DbgSrc, FileFind, SynHighlighterAsm, SynHighlighterCpp,
  SynEditHighlighter, SynHighlighterPas, JclDebug, AsmLst, ModulInf;


type
  TViewFrame = class(TFrame)
    Lv: TListView;
    Ed: TSynEdit;
    ErrPanel: TPanel;
    SynPasSyn1: TSynPasSyn;
    SynCppSyn1: TSynCppSyn;
    SynAsmSyn1: TSynAsmSyn;

    procedure EdGutterPaint(Sender: TObject; aLine, X, Y: Integer);
    procedure EdGutterGetText(Sender: TObject; aLine: Integer;
      var aText: String);
    procedure EdKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EdMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    CurrEditUnit: TUnitHits;
    ShowingAsm: boolean;

    procedure ShowModuleInfo(AModule: TModuleHits);
    function ShowUnitSrc(AUnit: TUnitHits; Silent: boolean = true): boolean;
    function ShowSrcPos(AItem: TCustomHitCounter): boolean;
    procedure ShowAsm(AAddr: DWord; AItem: TCustomHitCounter);
    procedure ShowEdit;
    procedure ShowList;
    procedure LineCountsToLines;
    procedure GetSourceCodeEvent(Sender: TObject; AModule: TModuleItem; AUnitName: string; ALines: TStringList);
    procedure GetSyncPointsEvent(Sender: TObject; AStartAddr: DWord; APoints: TIntegerArray);
    function FindSourceFile(AModuleInfo: TModuleItem; AUnitName: string): TFileName;
    procedure CountAsmHits;
  public
    Finder: TSourceFileFinder;
    Asmer: TAsmLister;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    procedure ShowErr(Msg: string);
    procedure ShowTotals;
    procedure ShowItem(AItem: TCustomHitCounter);
  end;



implementation

{$R *.DFM}

uses
  Main;


{ TViewFrame }

//------------------------------------------------------------------------------
//                                system
//------------------------------------------------------------------------------
constructor TViewFrame.Create(AOwner: TComponent);
begin
  inherited;

  Finder := TSourceFileFinder.Create;

  Lv.Align := alClient;
  Ed.Align := alClient;
end;


destructor TViewFrame.Destroy;
begin
  Finder.Free;
  Asmer.Free;
  inherited;  
end;


procedure TViewFrame.Clear;
begin
  Lv.Items.Clear;
  Ed.Lines.Clear;
  if Asmer <> nil then Asmer.Clear;

  CurrEditUnit := nil;

  MainForm.Panel2.Caption := '';
  MainForm.Panel2.Update;
  ShowErr('');
end;







//------------------------------------------------------------------------------
//                                GUI
//------------------------------------------------------------------------------
procedure TViewFrame.ShowErr(Msg: string);
begin
  ErrPanel.Caption := Msg;
  ErrPanel.Visible := true;
  Lv.Visible := false;
  Ed.Visible := false;
  ErrPanel.Update;
end;


procedure TViewFrame.ShowList;
begin
  Lv.Visible := true;
  ErrPanel.Visible := false;
  Ed.Visible := false;
end;


procedure TViewFrame.ShowEdit;
begin
  Ed.Visible := true;
  Lv.Visible := false;
  ErrPanel.Visible := false;
end;


procedure TViewFrame.EdGutterGetText(Sender: TObject; aLine: Integer;
  var aText: String);
begin
  if not ShowingAsm then Exit;
  if (aLine-1) > High(Asmer.Addresses) then aText := ''
  else if Asmer.Addresses[aLine-1] = 0 then aText := ''
  else aText := Format('%.8x', [Asmer.Addresses[aLine-1]]);
end;


procedure TViewFrame.EdGutterPaint(Sender: TObject; aLine, X, Y: Integer);
var
  Cnt: integer;
  S: string;
begin
  Cnt := Integer(Ed.Lines.Objects[aLine-1]);
  if Cnt = 0 then Exit;

  with Ed.Canvas do
    begin
    Brush.Style := bsClear;
    Font.Color := clRed;
    Font.Style := [fsBold];

    S := IntToStr(Cnt);
    Inc(X, Ed.GutterWidth - TextWidth(S) - 5);
    TextOut(X, Y+1, S);
    end;
end;


procedure TViewFrame.EdKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Ed.ActiveLineColor := clNone;
end;


procedure TViewFrame.EdMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Ed.ActiveLineColor := clNone;
end;







//------------------------------------------------------------------------------
//                                  show
//------------------------------------------------------------------------------
procedure TViewFrame.ShowTotals;
var
  Item: TListItem;
  Ex: Extended;
begin
  MainForm.Panel2.Caption := '';
  Lv.Items.Clear;

  Item := Lv.Items.Add;
  Item.Caption := 'Application';
  Item.SubItems.Add(MainForm.Settings.AppName);

  Item := Lv.Items.Add;
  Item.Caption := 'Total Samples';
  Ex := MainForm.Perf.TotalHits;
  Item.SubItems.Add(Format('%.0n', [Ex]));

  Item := Lv.Items.Add;
  Item.Caption := 'Process Samples';
  Ex := MainForm.Perf.ProcessHits;
  Item.SubItems.Add(Format('%.0n  (%1f%%)', [Ex,100 * MainForm.Perf.ProcessHits / MainForm.Perf.TotalHits]));


  ShowList;
end;


procedure TViewFrame.ShowItem(AItem: TCustomHitCounter);
begin
  //bin file info
  if AItem is TModuleHits then ShowModuleInfo(AItem as TModuleHits)
  //src file
  else if AItem is TUnitHits then ShowUnitSrc(AItem as TUnitHits, false)
  //asm code
  else if AItem is TAddrHits then ShowAsm((AItem as TAddrHits).Addr, nil)
  //func or line
  else if not ShowSrcPos(AItem) then ShowAsm(0, AItem);
end;






//------------------------------------------------------------------------------
//                             helper func
//------------------------------------------------------------------------------
function FormatSize(ASize: Extended): string;
begin
  Result := Format('%.0n bytes', [ASize]);
end;


function FormatAddrRange(AddrB, AddrE: DWord): string;
begin
  Result := Format('$%.8x - $%.8x  (%s)', [AddrB, AddrE, FormatSize(AddrE-AddrB)]);
end;






//------------------------------------------------------------------------------
//                             module info
//------------------------------------------------------------------------------
procedure TViewFrame.ShowModuleInfo(AModule: TModuleHits);
var
  Item: TListItem;
  Vi: TJclFileVersionInfo;
  SizeExt: Extended;
begin
  if AModule.ModuleInfo.BinFile = '' then
    begin
    MainForm.Panel2.Caption := AModule.ModuleInfo.Name;
    ShowErr('Unknown module');
    Exit;
    end;

  MainForm.Panel2.Caption := AModule.ModuleInfo.BinFile;

  Lv.Items.Clear;

  //image info

  Item := Lv.Items.Add;
  Item.Caption := 'Name';
  Item.SubItems.Add(AModule.ModuleInfo.Name);

  Item := Lv.Items.Add;
  Item.Caption := 'Address Range';
  Item.SubItems.Add(FormatAddrRange(AModule.ModuleInfo.BaseAddr, AModule.ModuleInfo.EndAddr));

{
  Item := Lv.Items.Add;
  Item.Caption := 'Code Sections';
  Item.SubItems.Add('');

  //code sections
  for i:=0 to High(AModule.ModuleInfo.Sections) do
    with  AModule.ModuleInfo.Sections[i] do
    begin
    Item := Lv.Items.Add;
    Item.Caption := '  ' + Name;
    Item.SubItems.Add(FormatAddrRange(StartAddr, EndAddr));
    end;
}

  Item := Lv.Items.Add;
  Item.Caption := 'Executed in';
  if AModule.Mode = 0
    then Item.SubItems.Add('User Mode')
    else Item.SubItems.Add('Kernel Mode');


  //file info
  Lv.Items.Add.SubItems.Add('');

  Item := Lv.Items.Add;
  Item.Caption := 'Location';
  Item.SubItems.Add(AModule.ModuleInfo.BinFile);

  Item := Lv.Items.Add;
  Item.Caption := 'File Exists';
  if AModule.ModuleInfo.BinExists
    then Item.SubItems.Add('Yes')
    else begin Item.SubItems.Add('No'); Exit; end;

  Item := Lv.Items.Add;
  Item.Caption := 'File Size';
  SizeExt := FileGetSize(AModule.ModuleInfo.BinFile);
  Item.SubItems.Add(FormatSize(SizeExt));

  Item := Lv.Items.Add;
  Item.Caption := 'Created';
  Item.SubItems.Add(FormatDateTime('yyyy-mm-dd hh:nn:ss', FileTimeToLocalDateTime(GetFileCreation(AModule.ModuleInfo.BinFile))));

  Item := Lv.Items.Add;
  Item.Caption := 'Last Modified';
  Item.SubItems.Add(FormatDateTime('yyyy-mm-dd hh:nn:ss', FileTimeToLocalDateTime(GetFileLastWrite(AModule.ModuleInfo.BinFile))));



  //version info
  try
    Vi := TJclFileVersionInfo.Create(AModule.ModuleInfo.BinFile);

    Lv.Items.Add.SubItems.Add('');

    Item := Lv.Items.Add;
    Item.Caption := 'Claimed Type';
    Item.SubItems.Add(OSFileTypeToString(Vi.FileType, Vi.FileSubType));

    Item := Lv.Items.Add;
    Item.Caption := 'Version';
    Item.SubItems.Add(Vi.FileVersion);

    Item := Lv.Items.Add;
    Item.Caption := 'Description';
    Item.SubItems.Add(Vi.FileDescription);

    Item := Lv.Items.Add;
    Item.Caption := 'Copyright';
    Item.SubItems.Add(Vi.LegalCopyright);

    Item := Lv.Items.Add;
    Item.Caption := 'Company';
    Item.SubItems.Add(Vi.CompanyName);

    Item := Lv.Items.Add;
    Item.Caption := 'Product';
    Item.SubItems.Add(Vi.ProductName);

    if Vi.Comments <> '' then
      begin
      Item := Lv.Items.Add;
      Item.Caption := 'Comments';
      Item.SubItems.Add(Vi.Comments);
      end;
  except end;


  //debug info
  Lv.Items.Add.SubItems.Add('');
  Item := Lv.Items.Add;
  Item.Caption := 'Debug Info';
  Item.SubItems.Add(DbgSInfoKindName[AModule.ModuleInfo.DebugInfoSource.Kind]);

  ShowList;
end;






//------------------------------------------------------------------------------
//                          unit source code
//------------------------------------------------------------------------------
function TViewFrame.ShowUnitSrc(AUnit: TUnitHits; Silent: boolean = true): boolean;
begin
  ShowingAsm := false;
  Ed.Gutter.DigitCount := 3;

  //get src file path
  if AUnit.SrcFile = '' then
    AUnit.SrcFile := FindSourceFile((AUnit.Parent as TModuleHits).ModuleInfo, AUnit.Name);

  //exit if not found
  Result := AUnit.SrcFile <> STR_NOTFOUND;
  if not Result then
    begin
    if not Silent then
      begin
      MainForm.Panel2.Caption := AUnit.Name;
      ShowErr('File could not be located');
      end;
    Exit;
    end;

  MainForm.Panel2.Caption := AUnit.SrcFile;

  //already loaded, exit
  if AUnit = CurrEditUnit then
    begin
    Ed.ActiveLineColor := clNone;
    Ed.CaretY := 1;
    ShowEdit;
    Exit;
    end;

  //file exists?
  Result := FileExists(AUnit.SrcFile);
  if not Result then
    begin
    CurrEditUnit := nil;
    if not Silent then ShowErr('File does not exist');
    Exit;
    end;

  //load file
  try
    //to do: cpp or pas
    if true then Ed.Highlighter := SynPasSyn1;

    CurrEditUnit := AUnit;

    Ed.Lines.LoadFromFile(AUnit.SrcFile);
    LineCountsToLines;
    Ed.ActiveLineColor := clNone;
    Ed.CaretY := 1;
    ShowEdit;
  except
    CurrEditUnit := nil;
    Result := false;
  end;

  //failed to load
  if not Result then
    begin
    CurrEditUnit := nil;
    if not Silent then ShowErr('Failed to load file');
    end;

{
  Item := Lv.Items.Add;
  Item.Caption := 'Name';
  Item.SubItems.Add(AUnit.Name);

  Item := Lv.Items.Add;
  Item.Caption := 'Location';
  Item.SubItems.Add(AUnit.SrcFile);

  Item := Lv.Items.Add;
  Item.Caption := 'Exists';
  if FileExists(AUnit.SrcFile)
    then Item.SubItems.Add('Yes')
    else begin Item.SubItems.Add('No'); Exit; end;

  Item := Lv.Items.Add;
  Item.Caption := 'Size';
  SizeExt := FileGetSize(AUnit.SrcFile);
  Item.SubItems.Add(FormatSize(SizeExt));

  Item := Lv.Items.Add;
  Item.Caption := 'Created';
  Item.SubItems.Add(FormatDateTime('yyyy-mm-dd hh:nn:ss', FileTimeToLocalDateTime(GetFileCreation(AUnit.SrcFile))));

  Item := Lv.Items.Add;
  Item.Caption := 'Last Modified';
  Item.SubItems.Add(FormatDateTime('yyyy-mm-dd hh:nn:ss', FileTimeToLocalDateTime(GetFileLastWrite(AUnit.SrcFile))));
}
end;


procedure TViewFrame.LineCountsToLines;
var
  i, j, Thr: integer;
begin
  if MainForm.ComboBox1.ItemIndex = -1
    then Thr := -1
    else Thr := MainForm.Perf.ThreadNumbers[MainForm.ComboBox1.ItemIndex];


  for i:=0 to CurrEditUnit.ChildCount-1 do
    with CurrEditUnit.Children[i] as TFuncHits do
      for j:=0 to ChildCount-1 do
        with Children[j] as TLineHits do
          if (LineNo > 0) and (LineNo <= Ed.Lines.Count) then
            if Thr < 0
              then Ed.Lines.Objects[LineNo-1] := Ptr(Hits)
              else Ed.Lines.Objects[LineNo-1] := Ptr(ThrHits[Thr]);
end;






//------------------------------------------------------------------------------
//                         func or line in src
//------------------------------------------------------------------------------
function TViewFrame.ShowSrcPos(AItem: TCustomHitCounter): boolean;
var
  SrcUnit: TCustomHitCounter;
  SrcLine: integer;
  LocInfo: TJclLocationInfo;
  Addr: DWord;
begin
  //show unit
  SrcUnit := AItem;
  while not (SrcUnit is TUnitHits) do SrcUnit := SrcUnit.Parent;
  Result := ShowUnitSrc(SrcUnit as TUnitHits);
  if not Result then Exit;
  
  //line # of item
  if AItem is TLineHits
    then //item is line
      SrcLine := (AItem as TLineHits).LineNo
    else //item is function
      begin
      Addr := AItem.FirstAddr;
      (SrcUnit.Parent as TModuleHits).ModuleInfo.DebugInfoSource.GetLocationInfo(Ptr(Addr), LocInfo);
      Dec(Addr, LocInfo.OffsetFromProcName);
      (SrcUnit.Parent as TModuleHits).ModuleInfo.DebugInfoSource.GetLocationInfo(Ptr(Addr), LocInfo);
      SrcLine := LocInfo.LineNumber;
      end;

  Result := SrcLine > 0;
  if not Result then Exit;

  //go to line
  Ed.CaretY := SrcLine;
  if (Ed.CaretY <= Ed.TopLine) or (Ed.CaretY > (Ed.TopLine + Ed.LinesInWindow - 3))
    then Ed.TopLine := Ed.CaretY - Ed.LinesInWindow div 3;
  Ed.ActiveLineColor := $00FFCACA;
end;






//------------------------------------------------------------------------------
//                                 asm
//------------------------------------------------------------------------------
procedure TViewFrame.ShowAsm(AAddr: DWord; AItem: TCustomHitCounter);
var
  Module: TCustomHitCounter;
  Dbg: TDebugInfoSource;
  LocInfo: TJclLocationInfo;
begin
  //get addr of item
  if (AItem <> nil) then
    begin
    Module := AItem;
    while not (Module is TModuleHits) do Module := Module.Parent;
    Dbg := (Module as TModuleHits).ModuleInfo.DebugInfoSource;

    if AItem.Name = STR_UNKNOWN then
      AAddr := AItem.FirstAddr

    else if AItem is TLineHits then
      begin
      AAddr := AItem.FirstAddr;
      Dbg.GetLocationInfo(Ptr(AAddr), LocInfo);
      Dec(AAddr, LocInfo.OffsetFromLineNumber);
      end

    else if AItem is TFuncHits then
      if Dbg.Kind = diBorSymb
        then
          begin
          AAddr := AItem.FirstAddr;
          Dbg.GetLocationInfo(Ptr(AAddr), LocInfo);
          Dec(AAddr, LocInfo.OffsetFromProcName);
          end
        else
          begin
          AAddr := Dbg.GetFuncAddress(AItem.Name);
          if AAddr = 0 then AAddr := AItem.FirstAddr;
          end;
    end;


  //disassemble
  if Asmer = nil then
    begin
    Asmer := TAsmLister.Create(MainForm.Perf.Modules);
    Asmer.OnGetSourceCode := GetSourceCodeEvent;
    Asmer.OnGetSyncPoints := GetSyncPointsEvent;
    end;

  if Asmer.MakeLines(AAddr, Ed.LinesInWindow * 3) or not ShowingAsm then
    begin
    //disasm changed
    MainForm.Panel2.Caption := Asmer.BinFile;
    Ed.Lines.Assign(Asmer.Lines);
    CountAsmHits;
    end
  else if Asmer.ErrText <> '' then
    //disasm failed
    begin ShowErr(Asmer.ErrText); Exit; end;


  //show if not showing
  if not ShowingAsm then
    begin
    Ed.Highlighter := SynAsmSyn1;
    Ed.Gutter.DigitCount := 8;
    CurrEditUnit := nil;
    ShowingAsm := true;
    end;

  //scroll to addr
  ShowEdit;
  Ed.CaretY := Asmer.LineAtAddr + 1;
  if (Ed.CaretY <= Ed.TopLine) or (Ed.CaretY > (Ed.TopLine + Ed.LinesInWindow - 3))
    then Ed.TopLine := Ed.CaretY - Ed.LinesInWindow div 3;
  Ed.ActiveLineColor := $00FFCACA;
end;


function TViewFrame.FindSourceFile(AModuleInfo: TModuleItem; AUnitName: string): TFileName;
begin
    if AModuleInfo.BinFile = MainForm.App.ExePath
      //use dpr if available
      then Finder.SetProject(MainForm.Settings.AppName)
      else Finder.SetProject(AModuleInfo.BinFile);

    Result := Finder.GetPath(AUnitName);
end;    


procedure TViewFrame.GetSourceCodeEvent(Sender: TObject;
  AModule: TModuleItem; AUnitName: string; ALines: TStringList);
begin
  try
    ALines.LoadFromFile(FindSourceFile(AModule, AUnitName));
  except
    ALines.Clear;
  end;
end;


procedure TViewFrame.CountAsmHits;
var
  Samples: TSamples;
  Thr, SampleNo, LineNo, i: integer;
  Addr: DWord;
begin
  if Ed.Lines.Count = 0 then Exit;

  if MainForm.ComboBox1.ItemIndex = -1
    then Thr := -1
    else Thr := MainForm.Perf.ThreadNumbers[MainForm.ComboBox1.ItemIndex];

  Samples := MainForm.Perf.Samples;

  //find first instruction in disasm listing
  LineNo := -1;
  for i:=0 to High(Asmer.Addresses) do
    if Asmer.Addresses[i] > 0 then
      begin LineNo := i; Break; end;
  if LineNo < 0 then Exit;

  //find the hit on or after the first instruction
  SampleNo := MainForm.Perf.FindSample(Asmer.Addresses[LineNo]);
  if SampleNo > High(Samples) then Exit;
  Addr := Samples[SampleNo].Addr;

  //count hits by instruction, filter by thread
  for LineNo:=LineNo to High(Asmer.Addresses) do
    if Asmer.Addresses[LineNo] = Addr then
      begin
      for SampleNo:=SampleNo to Length(Samples)do
        if SampleNo = Length(Samples) then Exit
        else if Samples[SampleNo].Addr <> Addr then Break
        else if (Thr < 0) or (Samples[SampleNo].Thr = Thr)
          then  Ed.Lines.Objects[LineNo] :=
            Ptr(Integer(Ed.Lines.Objects[LineNo]) +
            MainForm.Perf.Samples[SampleNo].Count);

      Addr := MainForm.Perf.Samples[SampleNo].Addr;
      end;
end;


//pass all sampled addresses to disasm as synch points 
procedure TViewFrame.GetSyncPointsEvent(Sender: TObject; AStartAddr: DWord;
  APoints: TIntegerArray);
var
  SampleNo, i: integer;
  EndAddr: DWord;
begin
  EndAddr := AStartAddr + Length(APoints);

  SampleNo := MainForm.Perf.FindSample(AStartAddr);

  for i:=SampleNo to High(MainForm.Perf.Samples) do
    if MainForm.Perf.Samples[i].Addr >= EndAddr then Break
    else Inc(APoints[MainForm.Perf.Samples[i].Addr - AStartAddr]);
end;



end.

